


// Define 'W3DDEBUG' to watch the debug string output.

// START OF W3D


#ifndef _W3D_DEFINITION_
#define _W3D_DEFINITION_


#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>
#include <math.h>
#include "w3dmath.h"
#include "dbllist.h"



//////////////////////////////////////////////////////////////////////////////
// W3D CLASS DEFINITIONS


// 0x0... _ERR_
// 0x1... _CLS_
// 0x2... _ATTRIB_


#define W3D_ERR_NOTINITIALIZED	"FATAL ERROR: W3D is not initialized!\n"

#define W3D_ERR_OK				0x00000000
#define W3D_ERR_FAIL			0x00000001

#define W3D_CLS_OBJECT			0x10000001
#define W3D_CLS_ATTRIB			0x10000002
#define W3D_CLS_MODEL			0x10000003
#define W3D_CLS_IMODEL			0x10000004
#define W3D_CLS_MODELER			0x10000005
#define W3D_CLS_SYSTEM			0x10000006
#define W3D_CLS_CAMERA			0x10000007

#define W3D_CLS_GROUP			0x10000011
#define W3D_CLS_MODELCOMPOSER	0x10000012
#define W3D_CLS_MODELRENDERER	0x10000013

#define	W3D_ATTRIB_TRANSLATE	0x20000001
#define	W3D_ATTRIB_SCALE		0x20000002
#define	W3D_ATTRIB_COLOR		0x20000003
#define	W3D_ATTRIB_ROTATE		0x20000004
#define W3D_ATTRIB_LOADIDENTITY	0x20000005


#define DECLARE_W3D_CLASS_TYPE(a)		virtual DWORD GetType() { return a; }

#define NAMESIZE	32




//////////////////////////////////////////////////////////////////////////////
// W3D CLASS FORWARD DECLARATIONS

class w3DObject;
class w3DCamera;
class w3DLoadIdentity;
class w3DTranslate;
class w3DRotate;
class w3DScale;
class w3DColor;
class w3DModelBase;
class w3DModeler;
class w3DSystem;
class w3DObjSphere;
class w3DObjCylinder;
class w3DIntegration;
class w3DGroup;


//////////////////////////////////////////////////////////////////////////////
// W3D HELPER FUNCTIONS PROTOTYPE FORWARD DECLARATIONS

void w3DContext(BOOL _ctx);
void w3DFlushAndSwap();
void w3DErrorTest(char *who);
void w3DMsgBox(const char *fmt, ...);
void w3DDebugMsg(const char *fmt, ...);
void w3DFree(void *ptr);
void w3DApplyAttrib(w3DGroup *pGrp);


//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DObject

class w3DObject
{
	char _name[NAMESIZE];	// object name

public:
	w3DObject();
	~w3DObject();

	char* GetName();
	void SetName(char *Name);

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_OBJECT)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DMalloc

class w3DMalloc
{
public:
	void	*pMalloc;
	DWORD	*pHeadDW, *pTailDW;
	DWORD	Size;
	DWORD	ActualSize;
	BOOL	Used;

	w3DMalloc();
	~w3DMalloc(); 

	void Free();
	void* Malloc(DWORD len);
	void Clean();

	static w3DMalloc* Bind(void* ptr);
	static void Free(void* ptr);
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DGroup

typedef LinkedList<w3DObject*> LISTW3D;

#define W3DGROUPNODETYPE	LISTW3D::Node

class w3DGroup : public w3DObject
{
private:
	LISTW3D					list;
	W3DGROUPNODETYPE		node;
	w3DGroup*				pParent;

public:
	w3DGroup();
	~w3DGroup();

	W3DGROUPNODETYPE* Add(w3DObject* item);
	void FindNext();
	void FindFirst();
	w3DObject* GetData();
	W3DGROUPNODETYPE& GetCurNode();
	void SetCurNode(W3DGROUPNODETYPE* pnode);
	BOOL DeleteFirst();
	void Empty();
	void SetParent(w3DGroup* pParent_);
	w3DGroup* GetParent();
	int GetTotalNum();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_GROUP)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DCamera

class w3DCamera : public w3DObject
{
private:
	float fieldofview, nearDist, farDist;
	w3DVector3 lu, lv, ln;
	w3DPoint3 le;

public:

	w3DPoint3 eye;
	w3DVector3 u, v, n;

	w3DCamera(float x, float y, float z, float _pitch, float _yaw, float _roll);
	w3DCamera();
	~w3DCamera();

	void Set(w3DPoint3 e, w3DPoint3 l, w3DVector3 up);   // eye, look, up
	void Set(float x, float y, float z, float _pitch, float _yaw, float _roll);
	void ResetAngle();
	void Yaw(float angle);
	void Pitch(float angle);
	void Roll(float angle);
	void Slide(float dU, float dV, float dN);

	void SetParam(float vAng, float nearD, float farD);
	float GetFOV();
	float GetNearDist();
	float GetFarDist();

	w3DPoint3 GetPos();
	float GetRoll();
	float GetYaw();

	void Apply();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_CAMERA)
};




//////////////////////////////////////////////////////////////////////////////
// DECLARATION & IMPLEMENTATION: w3DLoadIdentity

class w3DLoadIdentity : public w3DObject
{
public:
	w3DLoadIdentity() {}
	~w3DLoadIdentity() {}

	DECLARE_W3D_CLASS_TYPE(W3D_ATTRIB_LOADIDENTITY)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION & IMPLEMENTATION: w3DTranslate

class w3DTranslate : public w3DObject
{
public:
	GLfloat x, y, z;
	BOOL	apply;

public:
	w3DTranslate(GLfloat x, GLfloat y, GLfloat z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
		apply = TRUE;
	}
	w3DTranslate() : x(0), y(0), z(0), apply(TRUE) {}
	~w3DTranslate() {}

	void Set(GLfloat x, GLfloat y, GLfloat z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}

	void SetApply(BOOL a) {apply = a;}

	DECLARE_W3D_CLASS_TYPE(W3D_ATTRIB_TRANSLATE)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION & IMPLEMENTATION: w3DRotate

class w3DRotate : public w3DObject
{
public:
	GLfloat a, x, y, z;
	BOOL	apply;

public:
	w3DRotate(GLfloat a, GLfloat x, GLfloat y, GLfloat z)
	{
		this->a = a;
		this->x = x;
		this->y = y;
		this->z = z;
		apply = TRUE;
	}
	w3DRotate() : a(0), x(0), y(0), z(0), apply(TRUE) {}
	~w3DRotate() {}

	void Set(GLfloat a, GLfloat x, GLfloat y, GLfloat z)
	{
		this->a = a;
		this->x = x;
		this->y = y;
		this->z = z;
	}

	void SetApply(BOOL a) {apply = a;}

	DECLARE_W3D_CLASS_TYPE(W3D_ATTRIB_ROTATE)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION & IMPLEMENTATION: w3DScale

class w3DScale : public w3DObject
{
public:
	GLfloat x, y, z;

public:
	w3DScale(GLfloat x, GLfloat y, GLfloat z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}
	w3DScale() : x(0), y(0), z(0) {}
	~w3DScale() {}

	void Set(GLfloat x, GLfloat y, GLfloat z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}

	DECLARE_W3D_CLASS_TYPE(W3D_ATTRIB_SCALE)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION & IMPLEMENTATION: w3DColor

class w3DColor : public w3DObject
{
public:
	GLfloat rgba[4];

public:
	w3DColor(GLfloat rgba[4])
	{
		this->rgba[0] = rgba[0];
		this->rgba[1] = rgba[1];
		this->rgba[2] = rgba[2];
		this->rgba[3] = rgba[3];
	}
	w3DColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
	{
		this->rgba[0] = r;
		this->rgba[1] = g;
		this->rgba[2] = b;
		this->rgba[3] = a;
	}
	w3DColor() 
	{
		rgba[0] = rgba[1] = rgba[2] = rgba[3] = 0.0f;
	}
	~w3DColor() {}

	void Set(GLfloat rgba[4])
	{
		this->rgba[0] = rgba[0];
		this->rgba[1] = rgba[1];
		this->rgba[2] = rgba[2];
		this->rgba[3] = rgba[3];
	}
	void Set(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
	{
		this->rgba[0] = r;
		this->rgba[1] = g;
		this->rgba[2] = b;
		this->rgba[3] = a;
	}

	DECLARE_W3D_CLASS_TYPE(W3D_ATTRIB_COLOR)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DModel

class w3DModelBase : public w3DObject
{
private:
	w3DGroup* attr;

public:
	w3DModelBase();
	~w3DModelBase();
	virtual void Render();  // render the model

	void SetAttrib(w3DGroup *attr);
	w3DGroup* GetAttrib();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_MODEL)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DModeler

class w3DModeler : public w3DObject
{
protected:
	w3DGroup* model;

public:
	w3DModeler();
	~w3DModeler();

	virtual void Process();
	void SetModel(w3DGroup* model); 

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_MODELER)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DSystem

class w3DSystem : public w3DObject
{
private:
	BOOL	stop;

public:
	w3DModeler*		pModeler;
	w3DGroup*		apply_attr;	// immediately applyable attributes!
	w3DCamera*		pCamera;

	static const GLfloat PI;
	static const GLfloat TORAD;
	static const GLfloat TODEG;
	static const char* const Version;
	static GLfloat Aspect;
	static GLfloat bkColor[4];
	static BOOL	cull_face;
	static BOOL	PolygonMode;
	static GLfloat ambientLight[4];
	static GLfloat diffuseLight[4];
	static GLfloat specularLight[4];
	static GLfloat lightPos[4];
	static BYTE DepthBufferSize;
	static BYTE ColorDepth;

	static HWND		hWnd;
	static HDC		hDC;
	static HGLRC	hRC;
	static BOOL		bInitialized;
	static w3DCamera* ActiveCamera;


	w3DSystem();
	~w3DSystem();

	void Init(HWND hWnd);
	void AdjustCSInfo(CREATESTRUCT &cs);

	void SetModeler(w3DModeler *pMod);
	void SetActiveCamera(w3DCamera *pCam);
//	void SetInteraction();
//  void SetFileLoader();
	void ApplyAttrib(w3DGroup *attr);
	void PrepareRendering();
	void RenderScene();
	void Process();
	void Swap();
	void Stop();
	void Resume();
	void Resize(GLsizei w, GLsizei h);
	void SetupRC();
	
	
	static void SetAmbientLight(GLfloat r, GLfloat g, GLfloat b, GLfloat a);
	static void SetDiffuseLight(GLfloat r, GLfloat g, GLfloat b, GLfloat a);
	static void SetSpecularLight(GLfloat r, GLfloat g, GLfloat b, GLfloat a);
	static void SetLightPos(GLfloat x, GLfloat y, GLfloat z, GLfloat a);
	static void SetBackGroundColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a);
	static void SetCullFace(BOOL bi);
	static void SetPolygonMode(BOOL mode);
	static void SetColorDepth(BYTE cd);
	static void SetDepthBufferSize(BYTE db);
	static w3DCamera* GetActiveCamera();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_SYSTEM)
};



//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DObjSphere

class w3DObjSphere : public w3DModelBase
{
private:
	GLdouble radius;

public:
	w3DObjSphere(GLdouble r);
	~w3DObjSphere();

	void Render();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_MODEL)
};


//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DObjCylinder

class w3DObjCylinder : public w3DModelBase
{
private:
	GLdouble radius, height;

public:
	w3DObjCylinder(GLdouble r, GLdouble h);
	~w3DObjCylinder();

	void Render();

	DECLARE_W3D_CLASS_TYPE(W3D_CLS_MODEL)
};


//////////////////////////////////////////////////////////////////////////////
// DECLARATION: w3DIntegration

class w3DIntegration
{
public:
	w3DSystem	sys;
	w3DModeler	mod;
	w3DCamera	cam;

public:
	w3DIntegration();
	~w3DIntegration();

	void Init(HWND hwnd);

	DECLARE_W3D_CLASS_TYPE(NULL)
};


#endif
